package com.regexlab.ftp;

import java.io.*;
import java.net.*;

public class FtpServer
{
    public static void main(String argv[]) throws Exception
    {
        FtpServer svr = new FtpServer(Thread.currentThread().getThreadGroup());
        if(svr!=null)
        {
            if(argv.length > 0){
                svr.rootDir = argv[0].trim();
                if(svr.rootDir.charAt(svr.rootDir.length()-1)=='\\')
                    svr.rootDir += ".";
                
                if(argv.length > 1)
                {
                    svr.SERVER_PORT = Integer.parseInt(argv[1]);
                }
            }
            else
            {
                String cfgRoot = System.getProperty("ftp.root");
                if(cfgRoot != null && cfgRoot.length() != 0)
                {
                    svr.rootDir = cfgRoot;
                }
            }

            svr.RunFtpServer();
        }
    }

    ServerSocket    listenSocket     = null; // Socket
    private int     SERVER_PORT      =  21 ; // ˿
    boolean         keepRunning      = true;
    String          rootDir          = ".";
    ThreadGroup     group            = null;

    public FtpServer(ThreadGroup group) {
        this.group = group;
    }
    
    public void RunFtpServer() throws Exception
    {
        // 
        try{
            listenSocket=new ServerSocket(SERVER_PORT);
        }catch(IOException excpt){
            System.err.println("Sorry to open port "+SERVER_PORT+": "+excpt);
            System.exit(1);
        }

        System.out.println("Usage: ftpsvr [rootdir [port]]");
        System.out.println("Root = "+rootDir);
        System.out.println("Port = "+SERVER_PORT);
        Socket clientSocket=null;
        System.out.println("FTP Server started successfully");

        try
        {
            while(keepRunning)
            {
                // 
                clientSocket=listenSocket.accept();

                FtpThread oneFtpThread  = createFtpThread(clientSocket, new AnonymousLimit());

                // 
                oneFtpThread.start();
            }
        }
        catch(IOException excpt){
            System.err.println("Sorry ,Failed I/O:"+excpt);
        }
        finally{
            listenSocket.close();
        }
    }

    private FtpThread createFtpThread(Socket clientSocket, LimitInterface limit)
    {
        // ߳
        FtpThread oneFtpThread  = new FtpThread(clientSocket, group);

        limit.setServer(this);
        limit.setThread(oneFtpThread);

        oneFtpThread.theLimiter = limit;

        return oneFtpThread;
    }
    
    public void close() {
        this.keepRunning = false;
        try {
            this.listenSocket.close();
        }
        catch (IOException e) {
//            e.printStackTrace();
        }
    }

    public int getServerPort() {
        return SERVER_PORT;
    }

    public void setServerPort(int server_port) {
        SERVER_PORT = server_port;
    }
    
    public String getRootDir() {
        return rootDir;
    }

    public void setRootDir(String rootDir) {
        this.rootDir = rootDir;
    }

}

